/*
 *  Copyright (C) 2006-2015 Leandro Nini
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef OPENMPT_BACKEND_H
#define OPENMPT_BACKEND_H

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#ifdef HAVE_OPENMPT
#define LIBOPENMPT_ANCIENT_COMPILER
#include <libopenmpt/libopenmpt.hpp>

#include "inputBackend.h"

#include <fx.h>

#if defined(HAVE_MMAP) || defined(_WIN32)
#  include <FXMemMap.h>
#endif

/*****************************************************************/

typedef struct {
	FXint		channels;
	FXint		samplerate;
	FXint		resamplingMode;
	FXint		masterGain;
	FXint		stereoSeparation;
	FXint		volumeRamping;
} openmptConfig_t;

/*****************************************************************/

#include "configFrame.h"

class openmptConfig : public configFrame {
	FXDECLARE(openmptConfig)

private:
	FXDataTarget	_masterGain;
	FXDataTarget	_stereoSeparation;
	FXDataTarget	_volumeRamping;

private:
	openmptConfig() {};
	openmptConfig(const openmptConfig&);
	openmptConfig& operator=(const openmptConfig&);

public:
	openmptConfig(FXPacker* win);
	virtual ~openmptConfig() {};

public:
	enum{
		ID_FREQUENCY=FXDialogBox::ID_LAST,
		ID_CHANNELS,
		ID_RESAMPLING,
		ID_REVERB,
		ID_MEGABASS,
		ID_SURROUND,
		ID_LAST,
	};

public:
	long onCmdFrequency(FXObject*, FXSelector, void*);
	long onCmdChannels(FXObject*, FXSelector, void*);
	long onCmdResampling(FXObject*, FXSelector, void*);
};

/*****************************************************************/

class openmptBackend : public inputBackend {

	friend class openmptConfig;

private:
	static openmptConfig_t	_settings;

private:
	openmpt::module	*_module;

	bool		_unmo3;

	static FXString		_ext;

private:
	openmptBackend();

	/// Get temp file name
	const FXString tempFile(const FXString& fileName) {
		return FXSystem::getTempDirectory()+PATHSEP+FXPath::title(fileName)+".mod";
	}

	/// Delete temp file
	void delTempFile(const bool del, const FXString& tmpFile) {
		if (del) {
			FXTRACE((10, "Deleting temp file: %s\n", tmpFile.text()));
			FXFile::remove(tmpFile);
		}
	}

public:
	~openmptBackend();

	static const char name[];

	static bool init();

	/// Factory method
	static input* factory() { return new openmptBackend(); }

	/// Check if we support ext
	static bool supports(const char* fileName);

	/// Get supported extension
	FXString ext() const { return _ext; }

	/// Open file
	bool open(const FXString& fileName);

	/// Close file
	void close();

	/// Rewind to start
	bool rewind();

	/// Get samplerate
	unsigned int samplerate() const { return _settings.samplerate; }

	/// Get channels
	unsigned int channels() const { return _settings.channels; }

	/// Get precision
	sample_t precision() const { return SAMPLE_FLOAT; }

	/// Callback function
	size_t fillBuffer(void* buffer, const size_t bufferSize, const unsigned int seconds);

	/// Open config dialog
	void config(FXPacker* win) { new openmptConfig(win); }

	void loadSettings();

	void saveSettings();
};

#endif

#endif
