/*
 *  Copyright (C) 2006-2010 Leandro Nini
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "utils.h"

#ifdef _WIN32
#include <windows.h>

#ifdef UNICODE
const FXnchar* convertUtf(const FXString& string) {
	const FXint n=ncslen(string.text(), string.length());
	FXnchar* temp=new FXnchar[n+1];
	utf2ncs(temp, string.text(), string.length()+1);
	return temp;
}

const FXchar* convertNc(const FXnchar* string) {
	FXchar* temp=new FXchar[utfslen(string)];
	nc2utfs(temp, string);
	return temp;
}
#endif //UNICODE

#endif //_WIN32

void runCmd(const FXString& cmd) {
#ifdef _WIN32
	PROCESS_INFORMATION pi={0};
	STARTUPINFO si={sizeof(si)};
	si.dwFlags=STARTF_USESHOWWINDOW;
	si.wShowWindow=SW_HIDE;

#ifdef UNICODE
	FXnchar* command=const_cast<FXnchar*>(convertUtf(cmd));
#else
	FXchar* command=const_cast<FXchar*>(cmd.text());
#endif //UNICODE

	if (CreateProcess(0, command, NULL, NULL, FALSE, 0, NULL, NULL, &si, &pi)) {

		WaitForSingleObject(pi.hProcess, INFINITE);
		CloseHandle(pi.hProcess);
		CloseHandle(pi.hThread);
	}
#ifdef UNICODE
	delete [] command;
#endif //UNICODE
#else
	system(cmd.text());
#endif //_WIN32
}

#define MAX_CHARS	20

FXString shrink(const FXString& string) {

	return (string.count()>MAX_CHARS)?
		FXStringFormat("...%s", string.right(MAX_CHARS).text()):
		string;
}

/// Get gray value from RGBA color
#define FXGRAYVAL(rgba) (30*FXREDVAL(rgba)+59*FXGREENVAL(rgba)+11*FXBLUEVAL(rgba))/100

FXColor altColor(FXColor color) {

	FXint val=FXGRAYVAL(color);
	val=(FXint)(((val<128)?15.0:-15.0)*exp((255-val)/196.0));

	return FXRGB(FXCLAMP(0, FXREDVAL(color)+val, 255),
		FXCLAMP(0, FXGREENVAL(color)+val, 255),
		FXCLAMP(0, FXBLUEVAL(color)+val, 255));
}
